<?php
/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
# ***** BEGIN LICENSE BLOCK *****
# This file is part of InDefero, an open source project management application.
# Copyright (C) 2008 Céondo Ltd and contributors.
#
# InDefero is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# InDefero is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# ***** END LICENSE BLOCK ***** */

/**
 * A patch to be reviewed.
 *
 * A patch can be marked as being directly the commit, in that case
 * the patch does not store the diff file as it can be retrieved from
 * the backend.
 *
 */
class IDF_Review_Patch extends Pluf_Model
{
    public $_model = __CLASS__;

    function init()
    {
        $this->_a['table'] = 'idf_review_patches';
        $this->_a['model'] = __CLASS__;
        $this->_a['cols'] = array(
                             // It is mandatory to have an "id" column.
                            'id' =>
                            array(
                                  'type' => 'Pluf_DB_Field_Sequence',
                                  'blank' => true, 
                                  ),
                            'review' => 
                            array(
                                  'type' => 'Pluf_DB_Field_Foreignkey',
                                  'model' => 'IDF_Review',
                                  'blank' => false,
                                  'verbose' => __('review'),
                                  'relate_name' => 'patches',
                                  ),
                            'summary' =>
                            array(
                                  'type' => 'Pluf_DB_Field_Varchar',
                                  'blank' => false,
                                  'size' => 250,
                                  'verbose' => __('summary'),
                                  ),
                            'commit' => 
                            array(
                                  'type' => 'Pluf_DB_Field_Foreignkey',
                                  'model' => 'IDF_Commit',
                                  'blank' => false,
                                  'verbose' => __('commit'),
                                  'relate_name' => 'patches',
                                  ),
                            'description' =>
                            array(
                                  'type' => 'Pluf_DB_Field_Text',
                                  'blank' => false,
                                  'verbose' => __('description'),
                                  ),
                            'patch' =>
                            array(
                                  'type' => 'Pluf_DB_Field_File',
                                  'blank' => false,
                                  'verbose' => __('patch'),
                                  'help_text' => 'The patch is stored at the same place as the issue attachments with the same approach for the name.',
                                  ),
                            'creation_dtime' =>
                            array(
                                  'type' => 'Pluf_DB_Field_Datetime',
                                  'blank' => true,
                                  'verbose' => __('creation date'),
                                  'index' => true,
                                  ),
                            );
    }

    /**
     * Get the list of file comments.
     *
     * It will go through the patch comments and find for each the
     * file comments.
     *
     * @param array Filter to apply to the file comment list (array())
     */
    function getFileComments($filter=array())
    {
        $files = new ArrayObject();
        foreach ($this->get_comments_list(array('order'=>'creation_dtime ASC')) as $ct) {
            foreach ($ct->get_filecomments_list($filter) as $fc) {
                $files[] = $fc;
            }
        }
        return $files;
    }

    function _toIndex()
    {
        return '';
    }

    function preDelete()
    {
    }

    function preSave($create=false)
    {
        if ($create) {
            $this->creation_dtime = gmdate('Y-m-d H:i:s');
        }
    }

    function postSave($create=false)
    {
    }

    public function timelineFragment($request)
    {
        return '';
    }

    public function feedFragment($request)
    {
        return '';
    }
}
